'use strict';

var _require = require('./utils'),
    getAST = _require.getAST,
    getClassNames = _require.getClassNames;

var getGlamorStyleSheet = function getGlamorStyleSheet() {
  return require('glamor').styleSheet;
};

function valueMatches(declaration, value) {
  return value instanceof RegExp ? value.test(declaration.value) : value === declaration.value;
}

function toHaveStyleRule(received, property, value) {
  var selectors = getClassNames(received);
  var ast = getAST(selectors, getGlamorStyleSheet());

  var classNames = getClassNames(received);
  var rules = ast.stylesheet.rules.filter(function (rule) {
    return classNames.some(function (cn) {
      return rule.selectors.includes(`.${cn}`);
    });
  });

  var declaration = rules.reduce(function (decs, rule) {
    return Object.assign([], decs, rule.declarations);
  }, []).filter(function (dec) {
    return dec.type === 'declaration' && dec.property === property;
  }).pop();

  if (!declaration) {
    return {
      pass: false,
      message: function message() {
        return `Property not found: ${property}`;
      }
    };
  }

  var pass = valueMatches(declaration, value);

  var message = function message() {
    return `Expected ${property}${pass ? ' not ' : ' '}to match:\n` + `  ${value}\n` + 'Received:\n' + `  ${declaration.value}`;
  };

  return {
    pass,
    message
  };
}

module.exports = {
  toHaveStyleRule
};