'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var css = require('css');

function getNodes(node) {
  var nodes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  if (node.children) {
    var children = typeof node.children === 'function' ? node.children() : node.children;
    if (Array.isArray(children)) {
      children.forEach(function (child) {
        return getNodes(child, nodes);
      });
    }
  }

  if (typeof node === 'object') {
    nodes.push(node);
  }

  return nodes;
}

function markNodes(nodes) {
  nodes.forEach(function (node) {
    node.withStyles = true;
  });
}

function getSelectors(nodes) {
  return nodes.reduce(function (selectors, node) {
    var props = typeof node.props === 'function' ? node.props() : node.props;
    return getSelectorsFromProps(selectors, props);
  }, []);
}

function getSelectorsFromProps(selectors, props) {
  var className = props.className || props.class;
  if (className) {
    selectors = selectors.concat(className.toString().split(' ').map(function (cn) {
      return `.${cn}`;
    }));
  }
  var dataProps = Object.keys(props).reduce(function (dProps, key) {
    if (key.startsWith('data-')) {
      dProps.push(`[${key}]`);
    }
    return dProps;
  }, []);
  if (dataProps.length) {
    selectors = selectors.concat(dataProps);
  }
  return selectors;
}

function filterChildSelector(baseSelector) {
  if (baseSelector.slice(-1) === '>') {
    return baseSelector.slice(0, -1);
  }
  return baseSelector;
}

function getAST(nodeSelectors, styleSheet) {
  var tags = typeof styleSheet === 'function' ? styleSheet().tags : styleSheet.tags;
  var styles = tags.map(function (tag) {
    return (/* istanbul ignore next */tag.textContent || ''
    );
  }).join('\n');
  var ast = css.parse(styles);
  var rules = ast.stylesheet.rules.filter(filter);
  var mediaQueries = getMediaQueries(ast, filter);

  ast.stylesheet.rules = [].concat(_toConsumableArray(rules), _toConsumableArray(mediaQueries));

  return ast;

  function filter(rule) {
    if (rule.type === 'rule') {
      return rule.selectors.some(function (selector) {
        var baseSelector = filterChildSelector(selector.split(/:| |\./).filter(function (s) {
          return !!s;
        })[0]);
        return nodeSelectors.some(function (sel) {
          return sel === baseSelector || sel === `.${baseSelector}`;
        });
      });
    }
    return false;
  }
}

function getStyles(nodeSelectors, styleSheet) {
  var ast = getAST(nodeSelectors, styleSheet);
  var ret = css.stringify(ast);
  return ret;
}

function getMediaQueries(ast, filter) {
  return ast.stylesheet.rules.filter(function (rule) {
    return rule.type === 'media' || rule.type === 'supports';
  }).reduce(function (acc, mediaQuery) {
    mediaQuery.rules = mediaQuery.rules.filter(filter);

    if (mediaQuery.rules.length) {
      return acc.concat(mediaQuery);
    }

    return acc;
  }, []);
}

function shouldDive(node) {
  return typeof node.dive === 'function' && typeof node.type() !== 'string';
}

function isTagWithClassName(node) {
  return node.prop('className') && typeof node.type() === 'string';
}

function getClassNameFromEnzyme(received) {
  var tree = shouldDive(received) ? received.dive() : received;
  var components = tree.findWhere(isTagWithClassName);
  return components.length && components.first().prop('className');
}

function getClassNameFromTestRenderer(received) {
  return received.props.className || received.props.class;
}

function getClassNameFromCheerio(received) {
  if (received[0].type === 'root') {
    return received.children().first().attr('class');
  } else {
    return received.attr('class');
  }
}

// eslint-disable-next-line complexity
function getClassNames(received) {
  var className = void 0;

  if (received) {
    if (received.$$typeof === Symbol.for('react.test.json')) {
      className = getClassNameFromTestRenderer(received);
    } else if (typeof received.findWhere === 'function') {
      className = getClassNameFromEnzyme(received);
    } else if (received.cheerio === '[cheerio object]') {
      className = getClassNameFromCheerio(received);
    }
  }

  return className ? className.split(/\s/) : [];
}

module.exports = {
  getNodes,
  markNodes,
  getSelectors,
  getAST,
  getStyles,
  getClassNames
};